#include <cstdlib>
#include <cstring>
#include <string>
#include <vector>

using namespace std;

#include "loadmeter.hh"
#include "input.hh"
#include "inputline.hh"
#include "inputbutton.hh"

class Channel
{
public:
    typedef unsigned char WaveType[32];
    unsigned short Hz;
    unsigned char Volume;
    WaveType WaveForm;
    enum ModeType { TRI, Form, RND } Mode;
public:
    Channel() : Hz(0), Volume(0), Mode(Form)
    {
        SetDuty(50);
    }
    void SetDuty(unsigned percent)
    {
        unsigned limit = percent * 32 / 100;
        for(unsigned a=0; a<32; ++a)
            WaveForm[a] = a<limit ? 0 : 255;
    }
    inline void SetWave(const WaveType &wave)
    {
        memcpy(&WaveForm, &wave, sizeof(WaveForm));
    }
    inline void SetType(ModeType mode)
    {
        Mode = mode;
    }
    inline void SetHz(unsigned hz)
    {
        Hz = hz;
    }
    inline void SetVol(unsigned char vol)
    {
        Volume = vol;
    }
};

class ChannelState
{
    Channel ch[4];
public:
    inline Channel &operator[] (unsigned ind) { return ch[ind]; }
    inline const Channel &operator[] (unsigned ind) const { return ch[ind]; }
};

class Song
{
    unsigned char FormatVersion;
    unsigned char ChannelCount;
    unsigned char TickRate;
    vector<ChannelState> Rows;
public:
    Song() : FormatVersion(2), ChannelCount(4), TickRate(50) { }
    Song(const string &fn) : Rows()
    {
        if(!Load(fn))throw;
    }
    bool Load(const string &fn)
    {
        FILE *fp = fopen(fn.c_str(), "rb");
        if(!fp)return false;
        
        if(fgetc(fp) != 'S'
        || fgetc(fp) != 'N'
        || fgetc(fp) != 'D'
        || fgetc(fp) != 26) { fclose(fp); return false; }
        FormatVersion = fgetc(fp);
        ChannelCount  = fgetc(fp);
        TickRate      = fgetc(fp);
        
        ChannelState ch;
        
        ch[0].SetDuty(25);
        ch[1].SetDuty(25);
        ch[2].SetType(Channel::TRI);
        ch[3].SetType(Channel::RND);
        
        LoadMeter meter("Loading %s", fn.c_str());
        
        fseek(fp, 0, SEEK_END);
        
        long wholesize = ftell(fp);
        
        Rows.clear();
        
        fseek(fp, 0x10, SEEK_SET);
        for(;;)
        {
            meter.tick(ftell(fp), wholesize);
            
            int c = 0; /* Suppress a warning of unused var */
            int b = fgetc(fp);
            if(b == EOF)break;
            
            if(b == 0xFD) { c = fgetc(fp); }
            if(b == 0xFF) { b = 0xFD; c = 1; }
            if(b == 0xFD)
            {
                while(c > 0) { Rows.push_back(ch); --c; }
                continue;
            }
            if(b == 0xFE)
            {
                c = fgetc(fp);
                int t = fgetc(fp);
                ch[c].SetType(t==1 ? Channel::Form
                            : t==2 ? Channel::RND
                            : Channel::TRI);
            }
            else if(b == 0xFC)
            {
                c = fgetc(fp);
                Channel::WaveType wav;
                fread(&wav, 1, sizeof(wav), fp);
                ch[c].SetWave(wav);
            }
            else
            {
                c = b;
                b = fgetc(fp);
                ch[c].SetHz(b | (fgetc(fp) << 8));
                ch[c].SetVol(fgetc(fp));
            }
        }
        fclose(fp);
        return true;
    }
    inline unsigned size() const
    {
        return Rows.size();
    }
};

static int GetFn(string &fn, const string &title)
{
    InputGroup grp;
    InputLine *f = new InputLine(title.c_str());
    f->SetResult("mman2_9.snd");
    grp += f;
    grp += new InputButton(1, "OK");
    grp += new InputButton(0, "Cancel");
    if(!grp.run())return 0;
    fn = f->GetResult();
    return 1;
}

class Scroller
{
};

static void Interactive(Song &diu)
{
	
}

int main(void)
{
	string fn;
	if(!GetFn(fn, "Input filename: "))return 0;
    
    Song Diu(fn);
    
    Interactive(Diu);
    
    return 0;
}
